/*
 * gsampler-assistant.c
 * Copyright (C) Lilian Monchalin 2011 <isaya07@users.sf.net>
 * 
 * gsampler is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * gsampler is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <glib/gi18n-lib.h>

#include "gsampler-assistant.h"
#include "gsampler-option.h"
#include "gsampler-utils.h"

#define UI_FILE "gsampler-assistant.ui"

G_DEFINE_TYPE (GsamplerAssistant, gsampler_assistant, GTK_TYPE_ASSISTANT);

static void 
gsampler_assistant_prepare_event (GsamplerAssistant *self, GtkWidget *page, gpointer data)
{
	GsamplerOption *option = NULL;
	g_return_if_fail (self != NULL);

	if (gtk_assistant_get_current_page ((GtkAssistant*) self) == 2)
	{
		option = gsampler_option_get_instance ();
		gsampler_option_set_string (option, "Serveur", "serveur-com", 
		                            gtk_file_chooser_get_filename ((GtkFileChooser*) self->com_chooser));
		gsampler_option_set_string (option, "Serveur", "db-path", 
		                            gtk_file_chooser_get_filename ((GtkFileChooser*) self->db_chooser));
		gsampler_option_set_string (option, "Serveur", "serveur-host", 
		                            (gchar*) gtk_entry_get_text (self->host_entry));
		gsampler_option_set_string (option, "Serveur", "serveur-port", 
		                            (gchar*) gtk_entry_get_text (self->port_entry));
		gsampler_option_set_integer (option, "Serveur", "serveur-timeout", 
		                        	 gtk_spin_button_get_value_as_int (self->time_spin));
		gsampler_option_set_boolean (option, "Serveur", "serveur-autostart",
		                             gtk_toggle_button_get_active ((GtkToggleButton*) self->autostart_check));
		gsampler_option_set_integer (option, "Serveur", "serveur-wait", 
		                        	 gtk_spin_button_get_value_as_int (self->wait_spin));
		gsampler_option_save_keyfile (option);
		g_object_unref (option);
	}
}

static void
gsampler_assistant_cancel_event (GsamplerAssistant *self, gpointer data)
{
	g_return_if_fail (self != NULL);

	gtk_widget_destroy ((GtkWidget*) self);
}

static void
gsampler_assistant_init (GsamplerAssistant *self)
{
	GtkBuilder *builder = NULL;
	GError* error = NULL;
	GtkAlignment* align = NULL;
	GtkBox *intro_box;
	GtkWidget *intro_lbl;
	GtkBox *sum_box;
	GtkWidget *sum_lbl;
	gchar *load_file = NULL;
	GsamplerOption *option = NULL;

	load_file = gsampler_utils_get_data_file (UI_FILE);
	if (load_file != NULL)	 {
		builder = gtk_builder_new ();
		gtk_builder_add_from_file (builder, load_file, &error);
		if (error != NULL)
		{
			g_warning ("Couldn't load builder file: %s", error->message);
			g_error_free (error);
		}	
	
		/* Get the window object from the ui file */
		align = g_object_ref_sink (GTK_ALIGNMENT (gtk_builder_get_object (builder, "alignment")));
		self->db_chooser = g_object_ref_sink (GTK_FILE_CHOOSER_BUTTON (gtk_builder_get_object (builder, "db_chooser")));
		self->com_chooser = g_object_ref_sink (GTK_FILE_CHOOSER_BUTTON (gtk_builder_get_object (builder, "com_chooser")));
		self->host_entry = g_object_ref_sink (GTK_ENTRY (gtk_builder_get_object (builder, "host_entry")));
		self->port_entry = g_object_ref_sink (GTK_ENTRY (gtk_builder_get_object (builder, "port_entry")));
		self->time_spin = g_object_ref_sink (GTK_SPIN_BUTTON (gtk_builder_get_object (builder, "time_spin")));
		self->autostart_check = g_object_ref_sink (GTK_CHECK_BUTTON (gtk_builder_get_object (builder, "autostart_check")));
		self->wait_spin = g_object_ref_sink (GTK_SPIN_BUTTON (gtk_builder_get_object (builder, "wait_spin")));

		g_object_unref (builder);
		g_free (load_file);
		
		option = gsampler_option_get_instance ();
		gtk_file_chooser_set_filename ((GtkFileChooser*) self->com_chooser,
			                           gsampler_option_get_string (option, "Serveur", "serveur-com"));
		gtk_file_chooser_set_filename ((GtkFileChooser*) self->db_chooser,
			                           gsampler_option_get_string (option, "Serveur", "db-path"));
		gtk_entry_set_text (self->host_entry, 
			                gsampler_option_get_string (option, "Serveur", "serveur-host"));
		gtk_entry_set_text (self->port_entry, 
			                gsampler_option_get_string (option, "Serveur", "serveur-port"));
		gtk_spin_button_set_value (self->time_spin, 
			                       gsampler_option_get_integer (option, "Serveur", "serveur-timeout"));
		gtk_spin_button_set_value (self->wait_spin, 
			                       gsampler_option_get_integer (option, "Serveur", "serveur-wait"));
		gtk_toggle_button_set_active ((GtkToggleButton*) self->autostart_check, 
			                          gsampler_option_get_boolean (option, "Serveur", "serveur-autostart"));
		g_object_unref (option);
	
		intro_box = (GtkBox*) gtk_vbox_new (FALSE, 0);
		intro_lbl = gtk_label_new (_("This wizard will help you for setting Gsampler\n" \
			                       "Click the forward button to start the configuration."));
		gtk_box_pack_start (intro_box, intro_lbl, TRUE, TRUE, 5);

		gtk_assistant_append_page ((GtkAssistant*) self, (GtkWidget*) intro_box);
		gtk_assistant_set_page_title ((GtkAssistant*) self, (GtkWidget*) intro_box, 
			                          _("Gsampler first start"));
		gtk_assistant_set_page_type ((GtkAssistant*) self, (GtkWidget*) intro_box, 
			                         GTK_ASSISTANT_PAGE_INTRO);
		gtk_assistant_set_page_complete ((GtkAssistant*) self, (GtkWidget*) intro_box, TRUE);
		gtk_widget_show_all ((GtkWidget*) intro_box);
	
		gtk_assistant_append_page ((GtkAssistant*) self, (GtkWidget*) align);
		gtk_assistant_set_page_title ((GtkAssistant*) self, (GtkWidget*) align, 
			                          _("Configure the backend"));
		gtk_assistant_set_page_type ((GtkAssistant*) self, (GtkWidget*) align, 
			                         GTK_ASSISTANT_PAGE_PROGRESS);
		gtk_assistant_set_page_complete ((GtkAssistant*) self, (GtkWidget*) align, TRUE);
		gtk_widget_show_all ((GtkWidget*) align);

		sum_box = (GtkBox*) gtk_vbox_new (FALSE, 0);
		sum_lbl = gtk_label_new (_("Congratulation we can now start\n" \
			                       "A remember that this is an alpha soft."));
		gtk_box_pack_start (sum_box, sum_lbl, TRUE, TRUE, 5);

		gtk_assistant_append_page ((GtkAssistant*) self, (GtkWidget*) sum_box);
		gtk_assistant_set_page_title ((GtkAssistant*) self, (GtkWidget*) sum_box, 
			                          _("Gsampler first start done"));
		gtk_assistant_set_page_type ((GtkAssistant*) self, (GtkWidget*) sum_box, 
			                         GTK_ASSISTANT_PAGE_SUMMARY);
		gtk_assistant_set_page_complete ((GtkAssistant*) self, (GtkWidget*) sum_box, TRUE);
		gtk_widget_show_all ((GtkWidget*) sum_box);
	
		g_signal_connect (self, "prepare",
			                      (GCallback) gsampler_assistant_prepare_event, NULL);
		g_signal_connect (self, "cancel",
			              (GCallback) gsampler_assistant_cancel_event, NULL);
		g_object_unref (align);
	}
}

static void
gsampler_assistant_finalize (GObject *object)
{
	GsamplerAssistant *self = GSAMPLER_ASSISTANT (object);
	g_object_unref (self->db_chooser);
	g_object_unref (self->com_chooser);
	g_object_unref (self->host_entry);
	g_object_unref (self->port_entry);
	g_object_unref (self->time_spin);
	g_object_unref (self->autostart_check);
	g_object_unref (self->wait_spin);
	G_OBJECT_CLASS (gsampler_assistant_parent_class)->finalize (object);
}

static void
gsampler_assistant_class_init (GsamplerAssistantClass *klass)
{
	gsampler_assistant_parent_class = g_type_class_peek_parent (klass);
	G_OBJECT_CLASS (klass)->finalize = gsampler_assistant_finalize;
}

GsamplerAssistant* 
gsampler_assistant_new (void) 
{
	return g_object_new (GSAMPLER_TYPE_ASSISTANT, NULL);
}
